import { type ClientOptions, OpenAI } from "openai";
import { config, validateConfig } from "./config.js";

// Validate config on module load
validateConfig();

const clientOptions: ClientOptions = {
  apiKey: config.openai.apiKey,
  baseURL: config.openai.endpoint,
};
const openai = new OpenAI(clientOptions);

export async function openAI_ClassifyFeed(title: string): Promise<string> {
  if (!title || title.trim() === "") {
    throw new Error("Feed title is required for classification");
  }

  const prompt = `
以下のRSSフィードのタイトルを見て、適切なトピックカテゴリに分類してください。

フィードタイトル: ${title}

以下のカテゴリから1つを選択してください:
- テクノロジー
- ビジネス
- エンターテインメント
- スポーツ
- 科学
- 健康
- 政治
- 環境
- 教育
- その他

分類結果を上記カテゴリのいずれか1つだけ返してください。
`;

  try {
    const response = await openai.chat.completions.create({
      model: config.openai.modelName,
      messages: [{ role: "user", content: prompt.trim() }],
      temperature: 0.3,
    });

    const category = response.choices[0]?.message?.content?.trim();
    if (!category) {
      console.warn("OpenAI returned empty category, using default");
      return "その他";
    }

    return category;
  } catch (error) {
    console.error("Error classifying feed:", error);
    throw new Error(
      `Failed to classify feed: ${error instanceof Error ? error.message : "Unknown error"}`,
    );
  }
}

export async function openAI_GeneratePodcastContent(
  title: string,
  items: Array<{
    title: string;
    link: string;
    content?: string;
    description?: string;
  }>,
): Promise<string> {
  if (!title || title.trim() === "") {
    throw new Error("Feed title is required for podcast content generation");
  }

  if (!items || items.length === 0) {
    throw new Error(
      "At least one news item is required for podcast content generation",
    );
  }

  // Validate items
  const validItems = items.filter((item) => item.title && item.link);
  if (validItems.length === 0) {
    throw new Error("No valid news items found (title and link required)");
  }

  // Build detailed article information including content
  const articleDetails = validItems
    .map((item, i) => {
      let articleInfo = `${i + 1}. タイトル: ${item.title}\nURL: ${item.link}`;

      // Add content if available
      const content = item.content || item.description;
      if (content && content.trim()) {
        // Limit content length to avoid token limits
        const maxContentLength = 2000;
        const truncatedContent =
          content.length > maxContentLength
            ? content.substring(0, maxContentLength) + "..."
            : content;
        articleInfo += `\n内容: ${truncatedContent}`;
      }

      return articleInfo;
    })
    .join("\n\n");

  const prompt = `
あなたはプロのポッドキャスタです。以下に示すフィードタイトルに基づき、そのトピックに関する詳細なポッドキャスト原稿を作成してください。

以下の要件を満たしてください:
- もし英単語が含まれている場合は、**必ずすべてカタカナに変換**してください (例: "Google" → "グーグル")
- もし英語の文が含まれている場合は、すべて日本語に翻訳してください
- 各ニュース記事の具体的な内容を基に詳細な要約と解説を行ってください
- 約1000文字〜5000文字程度の長さにしてください
- 自然な日本語の口語表現を使ってください
- トピック全体のまとめで締めくくってください
- 各ニュース記事のタイトルを全て日本語に翻訳して、冒頭で紹介してください

記事の実際の内容を活用して、価値のあるポッドキャスト原稿を作成してください。
`;

  const sendContent = `
  フィードタイトル: ${title}
  関連するニュース記事:
${articleDetails}
`;

  try {
    const response = await openai.chat.completions.create({
      model: config.openai.modelName,
      messages: [{ role: "system", content: prompt.trim() }, {role:"user", content: sendContent.trim()}],
      temperature: 0.6,
    });

    const scriptText = response.choices[0]?.message?.content?.trim();
    if (!scriptText) {
      throw new Error("OpenAI returned empty podcast content");
    }

    return scriptText;
  } catch (error) {
    console.error("Error generating podcast content:", error);
    throw new Error(
      `Failed to generate podcast content: ${error instanceof Error ? error.message : "Unknown error"}`,
    );
  }
}

export async function openAI_ClassifyEpisode(
  title: string,
  description?: string,
  content?: string,
): Promise<string> {
  if (!title || title.trim() === "") {
    throw new Error("Episode title is required for classification");
  }

  // Build the text for classification based on available data
  let textForClassification = `タイトル: ${title}`;

  if (description && description.trim()) {
    textForClassification += `\n説明: ${description}`;
  }

  if (content && content.trim()) {
    const maxContentLength = 1500;
    const truncatedContent =
      content.length > maxContentLength
        ? content.substring(0, maxContentLength) + "..."
        : content;
    textForClassification += `\n内容: ${truncatedContent}`;
  }

  const prompt = `
ポッドキャストエピソードの情報を見て、適切なトピックカテゴリに分類してください。

以下のカテゴリから1つを選択してください:
- テクノロジー
- ビジネス
- エンターテインメント
- スポーツ
- 科学
- 健康
- 政治
- 環境
- 教育
- その他

エピソードの内容に最も適合するカテゴリを上記から1つだけ返してください。
`;

  try {
    const response = await openai.chat.completions.create({
      model: config.openai.modelName,
      messages: [{ role: "system", content: prompt.trim() }, {role: "user", content: textForClassification.trim()}],
      temperature: 0.3,
    });

    const category = response.choices[0]?.message?.content?.trim();
    if (!category) {
      console.warn("OpenAI returned empty episode category, using default");
      return "その他";
    }

    return category;
  } catch (error) {
    console.error("Error classifying episode:", error);
    throw new Error(
      `Failed to classify episode: ${error instanceof Error ? error.message : "Unknown error"}`,
    );
  }
}
